@@ -28,6 +28,8 @@
 package org.apache.http.impl.nio;
 
 import java.io.IOException;
+import java.net.InetSocketAddress;
+import java.net.SocketAddress;
 
 import org.apache.http.HttpEntity;
 import org.apache.http.HttpEntityEnclosingRequest;
@@ -160,6 +162,9 @@ public class DefaultNHttpClientConnection
                         this.connMetrics.incrementResponseCount();
                     }
                     handler.responseReceived(this);
+                    if (this.session.isClosed()) {
+                        return;
+                    }
                     if (this.contentDecoder == null) {
                         resetInput();
                     }
@@ -170,6 +175,9 @@ public class DefaultNHttpClientConnection
             }
             if (this.contentDecoder != null) {
                 handler.inputReady(this, this.contentDecoder);
+                if (this.session.isClosed()) {
+                    return;
+                }
                 if (this.contentDecoder.isCompleted()) {
                     // Response entity received
                     // Ready to receive a new response
@@ -254,17 +262,37 @@ public class DefaultNHttpClientConnection
         return this.request != null;
     }
 
-    @Override
     public String toString() {
-        StringBuffer buffer = new StringBuffer();
-        buffer.append("[");
-        if (isOpen()) {
-            buffer.append(this.session.getRemoteAddress());
-        } else {
-            buffer.append("closed");
+        StringBuilder buf = new StringBuilder();
+
+        final SocketAddress remoteAddress = this.session.getRemoteAddress();
+        final SocketAddress localAddress = this.session.getLocalAddress();
+
+        if (remoteAddress == null || localAddress == null) {
+            return "[CLOSED]";
         }
-        buffer.append("]");
-        return buffer.toString();
+
+        if (remoteAddress instanceof InetSocketAddress &&
+            localAddress instanceof InetSocketAddress) {
+
+            final InetSocketAddress remote = ((InetSocketAddress) remoteAddress);
+            final InetSocketAddress local  = ((InetSocketAddress) localAddress);
+
+            buf.append(local.getAddress() != null ? local.getAddress().getHostAddress() : local.getAddress())
+            .append(':')
+            .append(local.getPort())
+            .append("->")
+            .append(remote.getAddress() != null ? remote.getAddress().getHostAddress() : remote.getAddress())
+            .append(':')
+            .append(remote.getPort());
+
+        } else {
+            buf.append(localAddress)
+            .append("->")
+            .append(remoteAddress);
+        }
+
+        return buf.toString();
     }
 
 }
