@@ -89,8 +89,6 @@ import org.apache.http.protocol.HttpContext;
 public class NHttpConnectionBase
         implements NHttpConnection, HttpInetConnection, SessionBufferStatus {
 
-    protected final HttpContext context;
-
     protected final ContentLengthStrategy incomingContentStrategy;
     protected final ContentLengthStrategy outgoingContentStrategy;
 
@@ -101,7 +99,9 @@ public class NHttpConnectionBase
     protected final HttpTransportMetricsImpl outTransportMetrics;
     protected final HttpConnectionMetricsImpl connMetrics;
 
+    protected HttpContext context;
     protected IOSession session;
+    protected SocketAddress remote;
     protected volatile ContentDecoder contentDecoder;
     protected volatile boolean hasBufferedInput;
     protected volatile ContentEncoder contentEncoder;
@@ -129,8 +129,6 @@ public class NHttpConnectionBase
         if (params == null) {
             throw new IllegalArgumentException("HTTP params may not be null");
         }
-        this.session = session;
-        this.context = new SessionHttpContext(session);
 
         int buffersize = HttpConnectionParams.getSocketBufferSize(params);
         int linebuffersize = buffersize;
@@ -150,11 +148,17 @@ public class NHttpConnectionBase
                 this.inTransportMetrics,
                 this.outTransportMetrics);
 
-        this.session.setBufferStatus(this);
-        this.session.setEvent(EventMask.READ);
+        setSession(session);
         this.status = ACTIVE;
     }
 
+    private void setSession(final IOSession session) {
+        this.session = session;
+        this.context = new SessionHttpContext(this.session);
+        this.session.setBufferStatus(this);
+        this.remote = this.session.getRemoteAddress();
+    }
+
     /**
      * @since 4.1
      */
@@ -406,4 +410,13 @@ public class NHttpConnectionBase
         return this.connMetrics;
     }
 
+    @Override
+    public String toString() {
+        StringBuilder buf = new StringBuilder();
+        buf.append(this.session.getLocalAddress())
+            .append("<->")
+            .append(this.session.getRemoteAddress());
+        return buf.toString();
+    }
+
 }
