@@ -54,7 +54,7 @@ import org.apache.http.util.CharArrayBuffer;
  *
  * @deprecated use {@link DefaultHttpResponseParser}
  */
-@SuppressWarnings("unchecked")
+@SuppressWarnings("rawtypes")
 @Deprecated
 public class HttpResponseParser extends AbstractMessageParser {
 
