@@ -1,8 +1,4 @@
 /*
- * $HeadURL: https://svn.apache.org/repos/asf/httpcomponents/httpcore/tags/4.0.1/httpcore-nio/src/main/java/org/apache/http/impl/nio/reactor/AbstractMultiworkerIOReactor.java $
- * $Revision: 744539 $
- * $Date: 2009-02-14 18:23:26 +0100 (Sat, 14 Feb 2009) $
- *
  * ====================================================================
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -47,67 +43,74 @@ import java.util.List;
 import java.util.Set;
 import java.util.concurrent.ThreadFactory;
 
-import org.apache.http.nio.params.NIOReactorPNames;
 import org.apache.http.nio.params.NIOReactorParams;
 import org.apache.http.nio.reactor.IOEventDispatch;
 import org.apache.http.nio.reactor.IOReactor;
 import org.apache.http.nio.reactor.IOReactorException;
 import org.apache.http.nio.reactor.IOReactorExceptionHandler;
 import org.apache.http.nio.reactor.IOReactorStatus;
-import org.apache.http.params.CoreConnectionPNames;
 import org.apache.http.params.HttpConnectionParams;
 import org.apache.http.params.HttpParams;
 
 /**
- * Generic implementation of {@link IOReactor} that can run multiple 
- * {@link BaseIOReactor} instance in separate worker threads and distribute 
+ * Generic implementation of {@link IOReactor} that can run multiple
+ * {@link BaseIOReactor} instance in separate worker threads and distribute
  * newly created I/O session equally across those I/O reactors for a more
- * optimal resource utilization and a better I/O performance. Usually it is 
+ * optimal resource utilization and a better I/O performance. Usually it is
  * recommended to have one worker I/O reactor per physical CPU core.
  * <p>
  * <strong>Important note about exception handling</strong>
  * <p>
- * Protocol specific exceptions as well as those I/O exceptions thrown in the 
- * course of interaction with the session's channel are to be expected are to be 
- * dealt with by specific protocol handlers. These exceptions may result in 
- * termination of an individual session but should not affect the I/O reactor 
- * and all other active sessions. There are situations, however, when the I/O 
- * reactor itself encounters an internal problem such as an I/O exception in 
- * the underlying NIO classes or an unhandled runtime exception. Those types of 
- * exceptions are usually fatal and will cause the I/O reactor to shut down 
+ * Protocol specific exceptions as well as those I/O exceptions thrown in the
+ * course of interaction with the session's channel are to be expected are to be
+ * dealt with by specific protocol handlers. These exceptions may result in
+ * termination of an individual session but should not affect the I/O reactor
+ * and all other active sessions. There are situations, however, when the I/O
+ * reactor itself encounters an internal problem such as an I/O exception in
+ * the underlying NIO classes or an unhandled runtime exception. Those types of
+ * exceptions are usually fatal and will cause the I/O reactor to shut down
  * automatically.
  * <p>
- * There is a possibility to override this behavior and prevent I/O reactors 
- * from shutting down automatically in case of a runtime exception or an I/O 
- * exception in internal classes. This can be accomplished by providing a custom 
- * implementation of the {@link IOReactorExceptionHandler} interface. 
+ * There is a possibility to override this behavior and prevent I/O reactors
+ * from shutting down automatically in case of a runtime exception or an I/O
+ * exception in internal classes. This can be accomplished by providing a custom
+ * implementation of the {@link IOReactorExceptionHandler} interface.
  * <p>
- * If an I/O reactor is unable to automatically recover from an I/O or a runtime 
- * exception it will enter the shutdown mode. First off, it cancel all pending 
- * new session requests. Then it will attempt to close all active I/O sessions 
- * gracefully giving them some time to flush pending output data and terminate 
- * cleanly. Lastly, it will forcibly shut down those I/O sessions that still 
- * remain active after the grace period. This is a fairly complex process, where 
- * many things can fail at the same time and many different exceptions can be 
- * thrown in the course of the shutdown process. The I/O reactor will record all 
- * exceptions thrown during the shutdown process, including the original one 
- * that actually caused the shutdown in the first place, in an audit log. One 
+ * If an I/O reactor is unable to automatically recover from an I/O or a runtime
+ * exception it will enter the shutdown mode. First off, it cancel all pending
+ * new session requests. Then it will attempt to close all active I/O sessions
+ * gracefully giving them some time to flush pending output data and terminate
+ * cleanly. Lastly, it will forcibly shut down those I/O sessions that still
+ * remain active after the grace period. This is a fairly complex process, where
+ * many things can fail at the same time and many different exceptions can be
+ * thrown in the course of the shutdown process. The I/O reactor will record all
+ * exceptions thrown during the shutdown process, including the original one
+ * that actually caused the shutdown in the first place, in an audit log. One
  * can obtain the audit log using {@link #getAuditLog()}, examine exceptions
- * thrown by the I/O reactor prior and in the course of the reactor shutdown 
+ * thrown by the I/O reactor prior and in the course of the reactor shutdown
  * and decide whether it is safe to restart the I/O reactor.
- * 
- *
- * @version $Revision: 744539 $
+ * <p>
+ * The following parameters can be used to customize the behavior of this
+ * class:
+ * <ul>
+ *  <li>{@link org.apache.http.params.CoreConnectionPNames#TCP_NODELAY}</li>
+ *  <li>{@link org.apache.http.params.CoreConnectionPNames#SO_TIMEOUT}</li>
+ *  <li>{@link org.apache.http.params.CoreConnectionPNames#SO_LINGER}</li>
+ *  <li>{@link org.apache.http.nio.params.NIOReactorPNames#SELECT_INTERVAL}</li>
+ *  <li>{@link org.apache.http.nio.params.NIOReactorPNames#GRACE_PERIOD}</li>
+ *  <li>{@link org.apache.http.nio.params.NIOReactorPNames#INTEREST_OPS_QUEUEING}</li>
+ * </ul>
  *
  * @since 4.0
  */
 public abstract class AbstractMultiworkerIOReactor implements IOReactor {
 
     protected volatile IOReactorStatus status;
-    
+
     protected final HttpParams params;
     protected final Selector selector;
     protected final long selectTimeout;
+    protected final boolean interestOpsQueueing;
 
     private final int workerCount;
     private final ThreadFactory threadFactory;
@@ -115,23 +118,23 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
     private final Worker[] workers;
     private final Thread[] threads;
     private final Object statusLock;
-    
+
     protected IOReactorExceptionHandler exceptionHandler;
     protected List<ExceptionEvent> auditLog;
-    
+
     private int currentWorker = 0;
 
     /**
      * Creates an instance of AbstractMultiworkerIOReactor.
      *
-     * @param workerCount number of worker I/O reactors. 
-     * @param threadFactory the factory to create threads. 
+     * @param workerCount number of worker I/O reactors.
+     * @param threadFactory the factory to create threads.
      *   Can be <code>null</code>.
      * @param params HTTP parameters.
-     * @throws IOReactorException in case if a non-recoverable I/O error. 
+     * @throws IOReactorException in case if a non-recoverable I/O error.
      */
     public AbstractMultiworkerIOReactor(
-            int workerCount, 
+            int workerCount,
             final ThreadFactory threadFactory,
             final HttpParams params) throws IOReactorException {
         super();
@@ -148,6 +151,7 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
         }
         this.params = params;
         this.selectTimeout = NIOReactorParams.getSelectInterval(params);
+        this.interestOpsQueueing = NIOReactorParams.getInterestOpsQueueing(params);
         this.statusLock = new Object();
         this.workerCount = workerCount;
         if (threadFactory != null) {
@@ -164,11 +168,11 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
     public IOReactorStatus getStatus() {
         return this.status;
     }
-    
+
     /**
-     * Returns the audit log containing exceptions thrown by the I/O reactor 
+     * Returns the audit log containing exceptions thrown by the I/O reactor
      * prior and in the course of the reactor shutdown.
-     * 
+     *
      * @return audit log.
      */
     public synchronized List<ExceptionEvent> getAuditLog() {
@@ -182,9 +186,9 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
     /**
      * Adds the given {@link Throwable} object with the given time stamp
      * to the audit log.
-     * 
+     *
      * @param ex the exception thrown by the I/O reactor.
-     * @param timestamp the time stamp of the exception. Can be 
+     * @param timestamp the time stamp of the exception. Can be
      * <code>null</code> in which case the current date / time will be used.
      */
     protected synchronized void addExceptionEvent(final Throwable ex, Date timestamp) {
@@ -199,10 +203,10 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
         }
         this.auditLog.add(new ExceptionEvent(ex, timestamp));
     }
-    
+
     /**
      * Adds the given {@link Throwable} object to the audit log.
-     * 
+     *
      * @param ex the exception thrown by the I/O reactor.
      */
     protected void addExceptionEvent(final Throwable ex) {
@@ -211,59 +215,53 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
 
     /**
      * Sets exception handler for this I/O reactor.
-     * 
-     * @param exceptionHandler the exception handler. 
+     *
+     * @param exceptionHandler the exception handler.
      */
     public void setExceptionHandler(final IOReactorExceptionHandler exceptionHandler) {
         this.exceptionHandler = exceptionHandler;
     }
-    
+
     /**
      * Triggered to process I/O events registered by the main {@link Selector}.
      * <p>
      * Super-classes can implement this method to react to the event.
-     * 
+     *
      * @param count event count.
-     * @throws IOReactorException in case if a non-recoverable I/O error. 
+     * @throws IOReactorException in case if a non-recoverable I/O error.
      */
     protected abstract void processEvents(int count) throws IOReactorException;
-    
+
     /**
      * Triggered to cancel pending session requests.
      * <p>
      * Super-classes can implement this method to react to the event.
-     * 
-     * @throws IOReactorException in case if a non-recoverable I/O error. 
+     *
+     * @throws IOReactorException in case if a non-recoverable I/O error.
      */
     protected abstract void cancelRequests() throws IOReactorException;
-    
+
     /**
-     * Activates the main I/O reactor as well as all worker I/O reactors. 
-     * The I/O main reactor will start reacting to I/O events and triggering 
-     * notification methods. The worker I/O reactor in their turn will start 
-     * reacting to I/O events and dispatch I/O event notifications to the given 
+     * Activates the main I/O reactor as well as all worker I/O reactors.
+     * The I/O main reactor will start reacting to I/O events and triggering
+     * notification methods. The worker I/O reactor in their turn will start
+     * reacting to I/O events and dispatch I/O event notifications to the given
      * {@link IOEventDispatch} interface.
      * <p>
-     * This method will enter the infinite I/O select loop on 
-     * the {@link Selector} instance associated with this I/O reactor and used 
+     * This method will enter the infinite I/O select loop on
+     * the {@link Selector} instance associated with this I/O reactor and used
      * to manage creation of new I/O channels. Once a new I/O channel has been
-     * created the processing of I/O events on that channel will be delegated 
+     * created the processing of I/O events on that channel will be delegated
      * to one of the worker I/O reactors.
      * <p>
      * The method will remain blocked unto the I/O reactor is shut down or the
-     * execution thread is interrupted. 
-     * <p>
-     * The following HTTP parameters affect execution of this method:
-     * <p>
-     * The {@link NIOReactorPNames#SELECT_INTERVAL} parameter determines the 
-     * time interval in milliseconds at which the I/O reactor wakes up to check 
-     * for timed out sessions and session requests.
-     * 
+     * execution thread is interrupted.
+     *
      * @see #processEvents(int)
      * @see #cancelRequests()
-     * 
-     * @throws InterruptedIOException if the dispatch thread is interrupted. 
-     * @throws IOReactorException in case if a non-recoverable I/O error. 
+     *
+     * @throws InterruptedIOException if the dispatch thread is interrupted.
+     * @throws IOReactorException in case if a non-recoverable I/O error.
      */
     public void execute(
             final IOEventDispatch eventDispatch) throws InterruptedIOException, IOReactorException {
@@ -282,7 +280,7 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
             this.status = IOReactorStatus.ACTIVE;
             // Start I/O dispatchers
             for (int i = 0; i < this.dispatchers.length; i++) {
-                BaseIOReactor dispatcher = new BaseIOReactor(this.selectTimeout);
+                BaseIOReactor dispatcher = new BaseIOReactor(this.selectTimeout, this.interestOpsQueueing);
                 dispatcher.setExceptionHandler(exceptionHandler);
                 this.dispatchers[i] = dispatcher;
             }
@@ -300,7 +298,7 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
                 }
                 this.threads[i].start();
             }
-            
+
             for (;;) {
                 int readyCount;
                 try {
@@ -310,24 +308,24 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
                 } catch (IOException ex) {
                     throw new IOReactorException("Unexpected selector failure", ex);
                 }
-                
-                if (this.status.compareTo(IOReactorStatus.ACTIVE) > 0) {
-                    break;
+
+                if (this.status.compareTo(IOReactorStatus.ACTIVE) == 0) {
+                    processEvents(readyCount);
                 }
-                processEvents(readyCount);
 
                 // Verify I/O dispatchers
                 for (int i = 0; i < this.workerCount; i++) {
                     Worker worker = this.workers[i];
-                    Thread thread = this.threads[i];
-                    if (!thread.isAlive()) {
-                        Exception ex = worker.getException();
-                        if (ex != null) {
-                            throw new IOReactorException(
-                                    "I/O dispatch worker terminated abnormally", ex);
-                        }
+                    Exception ex = worker.getException();
+                    if (ex != null) {
+                        throw new IOReactorException(
+                                "I/O dispatch worker terminated abnormally", ex);
                     }
                 }
+
+                if (this.status.compareTo(IOReactorStatus.ACTIVE) > 0) {
+                    break;
+                }
             }
 
         } catch (ClosedSelectorException ex) {
@@ -338,8 +336,8 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
             }
             throw ex;
         } finally {
+            doShutdown();
             synchronized (this.statusLock) {
-                doShutdown();
                 this.status = IOReactorStatus.SHUT_DOWN;
                 this.statusLock.notifyAll();
             }
@@ -348,34 +346,30 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
 
     /**
      * Activates the shutdown sequence for this reactor. This method will cancel
-     * all pending session requests, close out all active I/O channels, 
+     * all pending session requests, close out all active I/O channels,
      * make an attempt to terminate all worker I/O reactors gracefully,
      * and finally force-terminate those I/O reactors that failed to
      * terminate after the specified grace period.
-     * <p>
-     * The following HTTP parameters affect execution of this method:
-     * <p>
-     * The {@link NIOReactorPNames#GRACE_PERIOD} parameter determines the grace 
-     * period the I/O reactors are expected to block waiting for individual 
-     * worker threads to terminate cleanly.
-     * 
+     *
      * @throws InterruptedIOException if the shutdown sequence has been
      *   interrupted.
      */
     protected void doShutdown() throws InterruptedIOException {
-        if (this.status.compareTo(IOReactorStatus.SHUTTING_DOWN) >= 0) {
-            return;
+        synchronized (this.statusLock) {
+            if (this.status.compareTo(IOReactorStatus.SHUTTING_DOWN) >= 0) {
+                return;
+            }
+            this.status = IOReactorStatus.SHUTTING_DOWN;
         }
-        this.status = IOReactorStatus.SHUTTING_DOWN;
         try {
-            cancelRequests();        
+            cancelRequests();
         } catch (IOReactorException ex) {
             if (ex.getCause() != null) {
                 addExceptionEvent(ex.getCause());
             }
         }
         this.selector.wakeup();
-        
+
         // Close out all channels
         if (this.selector.isOpen()) {
             Set<SelectionKey> keys = this.selector.keys();
@@ -405,7 +399,7 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
         }
 
         long gracePeriod = NIOReactorParams.getGracePeriod(this.params);
-        
+
         try {
             // Force shut down I/O dispatchers if they fail to terminate
             // in time
@@ -438,7 +432,7 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
 
     /**
      * Assigns the given channel entry to one of the worker I/O reactors.
-     *  
+     *
      * @param entry the channel entry.
      */
     protected void addChannel(final ChannelEntry entry) {
@@ -446,10 +440,10 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
         int i = Math.abs(this.currentWorker++ % this.workerCount);
         this.dispatchers[i].addChannel(entry);
     }
-    
+
     /**
      * Registers the given channel with the main {@link Selector}.
-     * 
+     *
      * @param channel the channel.
      * @param ops interest ops.
      * @return  selection key.
@@ -462,27 +456,7 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
 
     /**
      * Prepares the given {@link Socket} by resetting some of its properties.
-     * <p>
-     * The following HTTP parameters affect execution of this method:
-     * <p>
-     * {@link CoreConnectionPNames#TCP_NODELAY} parameter determines whether 
-     * Nagle's algorithm is to be used. The Nagle's algorithm tries to conserve 
-     * bandwidth by minimizing the number of segments that are sent. When 
-     * applications wish to decrease network latency and increase performance, 
-     * they can disable Nagle's algorithm (that is enable TCP_NODELAY). Data 
-     * will be sent earlier, at the cost of an increase in bandwidth 
-     * consumption.
-     * <p>
-     * {@link CoreConnectionPNames#SO_TIMEOUT} parameter defines the socket 
-     * timeout in milliseconds, which is the timeout for waiting for data. 
-     * A timeout value of zero is interpreted as an infinite timeout.
-     * <p>
-     * {@link CoreConnectionPNames#SO_LINGER} parameter defines linger time 
-     * in seconds. The maximum timeout value is platform specific. Value 
-     * <code>0</code> implies that the option is disabled. Value <code>-1</code>
-     * implies that the JRE default is to be used. The setting only affects 
-     * socket close.
-     *  
+     *
      * @param socket the socket
      * @throws IOException in case of an I/O error.
      */
@@ -496,11 +470,11 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
     }
 
     /**
-     * Blocks for the given period of time in milliseconds awaiting 
-     * the completion of the reactor shutdown. If the value of 
+     * Blocks for the given period of time in milliseconds awaiting
+     * the completion of the reactor shutdown. If the value of
      * <code>timeout</code> is set to <code>0</code> this method blocks
-     * indefinitely. 
-     *  
+     * indefinitely.
+     *
      * @param timeout the maximum wait time.
      * @throws InterruptedException if interrupted.
      */
@@ -529,12 +503,24 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
             if (this.status.compareTo(IOReactorStatus.ACTIVE) > 0) {
                 return;
             }
-            this.status = IOReactorStatus.SHUTDOWN_REQUEST;
-            this.selector.wakeup();
-            try {
-                awaitShutdown(waitMs);
-            } catch (InterruptedException ignore) {
+            if (this.status.compareTo(IOReactorStatus.INACTIVE) == 0) {
+                this.status = IOReactorStatus.SHUT_DOWN;
+                cancelRequests();
+                return;
             }
+            this.status = IOReactorStatus.SHUTDOWN_REQUEST;
+        }
+        this.selector.wakeup();
+        try {
+            awaitShutdown(waitMs);
+        } catch (InterruptedException ignore) {
+        }
+    }
+
+    static void closeChannel(final Channel channel) {
+        try {
+            channel.close();
+        } catch (IOException ignore) {
         }
     }
 
@@ -542,15 +528,15 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
 
         final BaseIOReactor dispatcher;
         final IOEventDispatch eventDispatch;
-        
+
         private volatile Exception exception;
-        
+
         public Worker(final BaseIOReactor dispatcher, final IOEventDispatch eventDispatch) {
             super();
             this.dispatcher = dispatcher;
             this.eventDispatch = eventDispatch;
         }
-        
+
         public void run() {
             try {
                 this.dispatcher.execute(this.eventDispatch);
@@ -558,7 +544,7 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
                 this.exception = ex;
             }
         }
-        
+
         public Exception getException() {
             return this.exception;
         }
@@ -568,11 +554,11 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
     static class DefaultThreadFactory implements ThreadFactory {
 
         private static volatile int COUNT = 0;
-        
+
         public Thread newThread(final Runnable r) {
             return new Thread(r, "I/O dispatcher " + (++COUNT));
         }
-        
+
     }
-    
+
 }
