@@ -1,8 +1,4 @@
 /*
- * $HeadURL: https://svn.apache.org/repos/asf/httpcomponents/httpcore/tags/4.0.1/httpcore-nio/src/main/java/org/apache/http/nio/protocol/AsyncNHttpClientHandler.java $
- * $Revision: 747992 $
- * $Date: 2009-02-26 04:03:22 +0100 (Thu, 26 Feb 2009) $
- *
  * ====================================================================
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -59,30 +55,36 @@ import org.apache.http.protocol.HttpContext;
 import org.apache.http.protocol.HttpProcessor;
 
 /**
- * Fully asynchronous HTTP client side protocol handler that implements the 
- * essential requirements of the HTTP protocol for the server side message 
- * processing as described by RFC 2616. It is capable of executing HTTP requests 
- * with nearly constant memory footprint. Only HTTP message heads are stored in 
- * memory, while content of message bodies is streamed directly from the entity 
+ * Fully asynchronous HTTP client side protocol handler that implements the
+ * essential requirements of the HTTP protocol for the server side message
+ * processing as described by RFC 2616. It is capable of executing HTTP requests
+ * with nearly constant memory footprint. Only HTTP message heads are stored in
+ * memory, while content of message bodies is streamed directly from the entity
  * to the underlying channel (and vice versa) using {@link ConsumingNHttpEntity}
  * and {@link ProducingNHttpEntity} interfaces.
- * 
- * When using this implementation, it is important to ensure that entities 
- * supplied for writing implement !ProducingNHttpEntity. Doing so will allow 
- * the entity to be written out asynchronously. If entities supplied for writing 
- * do not implement the {@link ProducingNHttpEntity} interface, a delegate is 
- * added that buffers the entire contents in memory. Additionally, the 
- * buffering might take place in the I/O dispatch thread, which could cause I/O 
- * to block temporarily. For best results, one must ensure that all entities 
- * set on {@link HttpRequest}s from {@link NHttpRequestExecutionHandler} 
+ *
+ * When using this implementation, it is important to ensure that entities
+ * supplied for writing implement {@link ProducingNHttpEntity}. Doing so will allow
+ * the entity to be written out asynchronously. If entities supplied for writing
+ * do not implement the {@link ProducingNHttpEntity} interface, a delegate is
+ * added that buffers the entire contents in memory. Additionally, the
+ * buffering might take place in the I/O dispatch thread, which could cause I/O
+ * to block temporarily. For best results, one must ensure that all entities
+ * set on {@link HttpRequest}s from {@link NHttpRequestExecutionHandler}
  * implement {@link ProducingNHttpEntity}.
- * 
- * If incoming responses enclose a content entity, 
- * {@link NHttpRequestExecutionHandler} are expected to return a 
- * {@link ConsumingNHttpEntity} for reading the content. After the entity is 
- * finished reading the data, 
- * {@link NHttpRequestExecutionHandler#handleResponse(HttpResponse, HttpContext)} 
- * method is called to process the response. 
+ *
+ * If incoming responses enclose a content entity,
+ * {@link NHttpRequestExecutionHandler} are expected to return a
+ * {@link ConsumingNHttpEntity} for reading the content. After the entity is
+ * finished reading the data,
+ * {@link NHttpRequestExecutionHandler#handleResponse(HttpResponse, HttpContext)}
+ * method is called to process the response.
+ * <p>
+ * The following parameters can be used to customize the behavior of this
+ * class:
+ * <ul>
+ *  <li>{@link org.apache.http.params.CoreProtocolPNames#WAIT_FOR_CONTINUE}</li>
+ * </ul>
  *
  * @see ConsumingNHttpEntity
  * @see ProducingNHttpEntity
@@ -163,7 +165,7 @@ public class AsyncNHttpClientHandler extends NHttpHandlerBase
             this.eventListener.fatalIOException(ex, conn);
         }
     }
-    
+
     public void requestReady(final NHttpClientConnection conn) {
         HttpContext context = conn.getContext();
 
@@ -184,10 +186,10 @@ public class AsyncNHttpClientHandler extends NHttpHandlerBase
 
             context.setAttribute(ExecutionContext.HTTP_REQUEST, request);
             this.httpProcessor.process(request, context);
-            
+
             HttpEntityEnclosingRequest entityReq = null;
             HttpEntity entity = null;
-            
+
             if (request instanceof HttpEntityEnclosingRequest) {
                 entityReq = (HttpEntityEnclosingRequest) request;
                 entity = entityReq.getEntity();
@@ -198,7 +200,7 @@ public class AsyncNHttpClientHandler extends NHttpHandlerBase
             } else if (entity != null) {
                 connState.setProducingEntity(new NHttpEntityWrapper(entity));
             }
-            
+
             connState.setRequest(request);
             conn.submitRequest(request);
             connState.setOutputState(ClientConnState.REQUEST_SENT);
@@ -210,6 +212,8 @@ public class AsyncNHttpClientHandler extends NHttpHandlerBase
                         CoreProtocolPNames.WAIT_FOR_CONTINUE, 3000);
                 conn.setSocketTimeout(timeout);
                 connState.setOutputState(ClientConnState.EXPECT_CONTINUE);
+            } else if (connState.getProducingEntity() != null) {
+                connState.setOutputState(ClientConnState.REQUEST_BODY_STREAM);
             }
 
         } catch (IOException ex) {
@@ -264,6 +268,9 @@ public class AsyncNHttpClientHandler extends NHttpHandlerBase
             ProducingNHttpEntity entity = connState.getProducingEntity();
 
             entity.produceContent(encoder, conn);
+            if (encoder.isCompleted()) {
+                connState.setOutputState(ClientConnState.REQUEST_BODY_DONE);
+            }
         } catch (IOException ex) {
             shutdownConnection(conn, ex);
             if (this.eventListener != null) {
@@ -293,14 +300,18 @@ public class AsyncNHttpClientHandler extends NHttpHandlerBase
                 return;
             } else {
                 connState.setResponse(response);
-
                 if (connState.getOutputState() == ClientConnState.EXPECT_CONTINUE) {
                     cancelRequest(conn, connState);
+                } else if (connState.getOutputState() == ClientConnState.REQUEST_BODY_STREAM) {
+                    // Early response
+                    cancelRequest(conn, connState);
+                    connState.invalidate();
+                    conn.suspendOutput();
                 }
             }
 
             context.setAttribute(ExecutionContext.HTTP_RESPONSE, response);
-            
+
             if (!canResponseHaveBody(request, response)) {
                 conn.resetInput();
                 response.setEntity(null);
@@ -366,7 +377,7 @@ public class AsyncNHttpClientHandler extends NHttpHandlerBase
     }
 
     /**
-     * @throws IOException - not thrown currently 
+     * @throws IOException - not thrown currently
      */
     private void continueRequest(
             final NHttpClientConnection conn,
@@ -376,7 +387,7 @@ public class AsyncNHttpClientHandler extends NHttpHandlerBase
         conn.setSocketTimeout(timeout);
 
         conn.requestOutput();
-        connState.setOutputState(ClientConnState.REQUEST_SENT);
+        connState.setOutputState(ClientConnState.REQUEST_BODY_STREAM);
     }
 
     private void cancelRequest(
@@ -391,20 +402,24 @@ public class AsyncNHttpClientHandler extends NHttpHandlerBase
     }
 
     /**
-     * @throws HttpException - not thrown currently 
+     * @throws HttpException - not thrown currently
      */
     private void processResponse(
             final NHttpClientConnection conn,
             final ClientConnState connState) throws IOException, HttpException {
 
+        if (!connState.isValid()) {
+            conn.close();
+        }
+
         HttpContext context = conn.getContext();
         HttpResponse response = connState.getResponse();
-
         this.execHandler.handleResponse(response, context);
-
         if (!this.connStrategy.keepAlive(response, context)) {
             conn.close();
-        } else {
+        }
+
+        if (conn.isOpen()) {
             // Ready for another request
             connState.resetInput();
             connState.resetOutput();
@@ -429,9 +444,14 @@ public class AsyncNHttpClientHandler extends NHttpHandlerBase
         private HttpResponse response;
         private ConsumingNHttpEntity consumingEntity;
         private ProducingNHttpEntity producingEntity;
-
+        private boolean valid;
         private int timeout;
 
+        public ClientConnState() {
+            super();
+            this.valid = true;
+        }
+
         public void setConsumingEntity(final ConsumingNHttpEntity consumingEntity) {
             this.consumingEntity = consumingEntity;
         }
@@ -501,6 +521,15 @@ public class AsyncNHttpClientHandler extends NHttpHandlerBase
             resetInput();
             resetOutput();
         }
+
+        public boolean isValid() {
+            return this.valid;
+        }
+
+        public void invalidate() {
+            this.valid = false;
+        }
+
     }
 
 }
