@@ -1,7 +1,4 @@
 /*
- * $HeadURL: https://svn.apache.org/repos/asf/httpcomponents/httpcore/tags/4.0.1/httpcore-nio/src/test/java/org/apache/http/impl/nio/codecs/TestHttpMessageParser.java $
- * $Revision: 744515 $
- * $Date: 2009-02-14 17:36:56 +0100 (Sat, 14 Feb 2009) $
  * ====================================================================
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -36,18 +33,18 @@ import java.io.UnsupportedEncodingException;
 import java.nio.channels.Channels;
 import java.nio.channels.ReadableByteChannel;
 
-import junit.framework.Test;
 import junit.framework.TestCase;
-import junit.framework.TestSuite;
 
 import org.apache.http.HttpException;
 import org.apache.http.HttpRequest;
 import org.apache.http.HttpRequestFactory;
+import org.apache.http.HttpResponse;
 import org.apache.http.HttpResponseFactory;
 import org.apache.http.HttpVersion;
 import org.apache.http.impl.DefaultHttpRequestFactory;
 import org.apache.http.impl.DefaultHttpResponseFactory;
 import org.apache.http.impl.nio.reactor.SessionInputBufferImpl;
+import org.apache.http.nio.NHttpMessageParser;
 import org.apache.http.nio.reactor.SessionInputBuffer;
 import org.apache.http.params.BasicHttpParams;
 import org.apache.http.params.CoreConnectionPNames;
@@ -55,9 +52,6 @@ import org.apache.http.params.HttpParams;
 
 /**
  * Simple tests for {@link AbstractMessageParser}.
- *
- * 
- * @version $Id: TestHttpMessageParser.java 744515 2009-02-14 16:36:56Z sebb $
  */
 public class TestHttpMessageParser extends TestCase {
 
@@ -66,35 +60,25 @@ public class TestHttpMessageParser extends TestCase {
         super(testName);
     }
 
-    // ------------------------------------------------------------------- Main
-    public static void main(String args[]) {
-        String[] testCaseName = { TestHttpMessageParser.class.getName() };
-        junit.textui.TestRunner.main(testCaseName);
-    }
-
     // ------------------------------------------------------- TestCase Methods
 
-    public static Test suite() {
-        return new TestSuite(TestHttpMessageParser.class);
-    }
-
-    private static ReadableByteChannel newChannel(final String s, final String charset) 
+    private static ReadableByteChannel newChannel(final String s, final String charset)
             throws UnsupportedEncodingException {
         return Channels.newChannel(new ByteArrayInputStream(s.getBytes(charset)));
     }
-    
-    private static ReadableByteChannel newChannel(final String s) 
+
+    private static ReadableByteChannel newChannel(final String s)
             throws UnsupportedEncodingException {
         return newChannel(s, "US-ASCII");
     }
 
     public void testSimpleParsing() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
         requestParser.fillBuffer(newChannel("GET /whatever HTTP/1.1\r\nSome header: stuff\r\n\r\n"));
-        HttpRequest request = (HttpRequest) requestParser.parse();
+        HttpRequest request = requestParser.parse();
         assertNotNull(request);
         assertEquals("/whatever", request.getRequestLine().getUri());
         assertEquals(1, request.getAllHeaders().length);
@@ -102,147 +86,147 @@ public class TestHttpMessageParser extends TestCase {
 
     public void testParsingChunkedMessages() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
 
         requestParser.fillBuffer(newChannel("GET /whatev"));
-        HttpRequest request = (HttpRequest) requestParser.parse();
+        HttpRequest request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("er HTTP/1.1\r"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("\nSome header: stuff\r\n\r\n"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
 
         assertNotNull(request);
         assertEquals("/whatever", request.getRequestLine().getUri());
         assertEquals(1, request.getAllHeaders().length);
-        
+
     }
 
     public void testParsingFoldedHeaders() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
 
         requestParser.fillBuffer(newChannel("GET /whatev"));
-        HttpRequest request = (HttpRequest) requestParser.parse();
+        HttpRequest request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("er HTTP/1.1\r"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("\nSome header: stuff\r\n"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("   more\r\n"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("\tstuff\r\n"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("\r\n"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
 
         assertNotNull(request);
         assertEquals("/whatever", request.getRequestLine().getUri());
         assertEquals(1, request.getAllHeaders().length);
         assertEquals("stuff more stuff", request.getFirstHeader("Some header").getValue());
     }
-    
+
     public void testParsingBadlyFoldedFirstHeader() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
 
         requestParser.fillBuffer(newChannel("GET /whatev"));
-        HttpRequest request = (HttpRequest) requestParser.parse();
+        HttpRequest request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("er HTTP/1.1\r"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("\n  Some header: stuff\r\n"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("   more stuff\r\n"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("\r\n"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
 
         assertNotNull(request);
         assertEquals("/whatever", request.getRequestLine().getUri());
         assertEquals(1, request.getAllHeaders().length);
         assertEquals("stuff more stuff", request.getFirstHeader("Some header").getValue());
     }
-    
+
     public void testParsingEmptyFoldedHeader() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
 
         requestParser.fillBuffer(newChannel("GET /whatev"));
-        HttpRequest request = (HttpRequest) requestParser.parse();
+        HttpRequest request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("er HTTP/1.1\r"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("\n  Some header: stuff\r\n"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("      \r\n"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("      more stuff\r\n"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
         assertNull(request);
         requestParser.fillBuffer(newChannel("\r\n"));
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
 
         assertNotNull(request);
         assertEquals("/whatever", request.getRequestLine().getUri());
         assertEquals(1, request.getAllHeaders().length);
         assertEquals("stuff  more stuff", request.getFirstHeader("Some header").getValue());
     }
-    
+
     public void testParsingIncompleteRequestLine() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
-        
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
+
         ReadableByteChannel channel = newChannel("GET /whatever HTTP/1.0");
         requestParser.fillBuffer(channel);
         requestParser.fillBuffer(channel);
-        HttpRequest request = (HttpRequest) requestParser.parse();
+        HttpRequest request = requestParser.parse();
         assertNotNull(request);
         assertEquals(HttpVersion.HTTP_1_0, request.getRequestLine().getProtocolVersion());
     }
-    
+
     public void testParsingIncompleteHeader() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
-        
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
+
         ReadableByteChannel channel = newChannel("GET /whatever HTTP/1.0\r\nHeader: whatever");
         requestParser.fillBuffer(channel);
         requestParser.fillBuffer(channel);
-        HttpRequest request = (HttpRequest) requestParser.parse();
+        HttpRequest request = requestParser.parse();
         assertNotNull(request);
         assertEquals(1, request.getAllHeaders().length);
         assertEquals("whatever", request.getFirstHeader("Header").getValue());
     }
-    
+
     public void testParsingInvalidRequestLine() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
-        
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
+
         ReadableByteChannel channel = newChannel("GET garbage\r\n");
         requestParser.fillBuffer(channel);
         try {
@@ -252,13 +236,13 @@ public class TestHttpMessageParser extends TestCase {
             // expected
         }
     }
-    
+
     public void testParsingInvalidStatusLine() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpResponseFactory responseFactory = new DefaultHttpResponseFactory();
-        HttpResponseParser responseParser = new HttpResponseParser(inbuf, null, responseFactory, params);
-        
+        NHttpMessageParser<HttpResponse> responseParser = new DefaultHttpResponseParser(inbuf, null, responseFactory, params);
+
         ReadableByteChannel channel = newChannel("HTTP 200 OK\r\n");
         responseParser.fillBuffer(channel);
         try {
@@ -268,13 +252,13 @@ public class TestHttpMessageParser extends TestCase {
             // expected
         }
     }
-    
+
     public void testParsingInvalidHeader() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpResponseFactory responseFactory = new DefaultHttpResponseFactory();
-        HttpResponseParser responseParser = new HttpResponseParser(inbuf, null, responseFactory, params);
-        
+        NHttpMessageParser<HttpResponse> responseParser = new DefaultHttpResponseParser(inbuf, null, responseFactory, params);
+
         ReadableByteChannel channel = newChannel("HTTP/1.0 200 OK\r\nstuff\r\n\r\n");
         responseParser.fillBuffer(channel);
         try {
@@ -284,53 +268,53 @@ public class TestHttpMessageParser extends TestCase {
             // expected
         }
     }
-    
+
     public void testResetParser() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
-        
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
+
         ReadableByteChannel channel = newChannel("GET /whatever HTTP/1.0\r\nHeader: one\r\n\r\n");
         requestParser.fillBuffer(channel);
-        HttpRequest request = (HttpRequest) requestParser.parse();
+        HttpRequest request = requestParser.parse();
         assertNotNull(request);
         assertEquals(1, request.getAllHeaders().length);
         assertEquals("one", request.getFirstHeader("Header").getValue());
-        
+
         requestParser.reset();
 
         channel = newChannel("GET /whatever HTTP/1.0\r\nHeader: two\r\n\r\n");
         requestParser.fillBuffer(channel);
-        request = (HttpRequest) requestParser.parse();
+        request = requestParser.parse();
         assertNotNull(request);
         assertEquals(1, request.getAllHeaders().length);
         assertEquals("two", request.getFirstHeader("Header").getValue());
     }
-    
+
     public void testInvalidConstructor() {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         try {
-            new HttpRequestParser(null, null, null, params);
+            new DefaultHttpRequestParser(null, null, null, params);
             fail("IllegalArgumentException should have been thrown");
         } catch (IllegalArgumentException ex) {
             // ignore
         }
         try {
-            new HttpRequestParser(inbuf, null, null, params);
+            new DefaultHttpRequestParser(inbuf, null, null, params);
             fail("IllegalArgumentException should have been thrown");
         } catch (IllegalArgumentException ex) {
             // ignore
         }
         try {
-            new HttpResponseParser(null, null, null, params);
+            new DefaultHttpResponseParser(null, null, null, params);
             fail("IllegalArgumentException should have been thrown");
         } catch (IllegalArgumentException ex) {
             // ignore
         }
         try {
-            new HttpResponseParser(inbuf, null, null, params);
+            new DefaultHttpResponseParser(inbuf, null, null, params);
             fail("IllegalArgumentException should have been thrown");
         } catch (IllegalArgumentException ex) {
             // ignore
@@ -339,17 +323,17 @@ public class TestHttpMessageParser extends TestCase {
 
     public void testLineLimitForStatus() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
 
         params.setIntParameter(CoreConnectionPNames.MAX_LINE_LENGTH, 0);
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
         requestParser.fillBuffer(newChannel("GET /whatever HTTP/1.0\r\nHeader: one\r\n\r\n"));
         requestParser.parse();
         requestParser.reset();
 
         params.setIntParameter(CoreConnectionPNames.MAX_LINE_LENGTH, 15);
-        requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
+        requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
         try {
             requestParser.fillBuffer(newChannel("GET /loooooooooooooooong HTTP/1.0\r\nHeader: one\r\n\r\n"));
             requestParser.parse();
@@ -360,17 +344,17 @@ public class TestHttpMessageParser extends TestCase {
 
     public void testLineLimitForHeader() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
 
         params.setIntParameter(CoreConnectionPNames.MAX_LINE_LENGTH, 0);
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
         requestParser.fillBuffer(newChannel("GET /whatever HTTP/1.0\r\nHeader: one\r\n\r\n"));
         requestParser.parse();
         requestParser.reset();
 
         params.setIntParameter(CoreConnectionPNames.MAX_LINE_LENGTH, 15);
-        requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
+        requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
         requestParser.fillBuffer(newChannel("GET / HTTP/1.0\r\nHeader: 9012345\r\n\r\n"));
         requestParser.parse();
         requestParser.reset();
@@ -384,12 +368,12 @@ public class TestHttpMessageParser extends TestCase {
 
     public void testLineLimitForFoldedHeader() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
 
         params.setIntParameter(CoreConnectionPNames.MAX_HEADER_COUNT, 2);
-        params.setIntParameter(CoreConnectionPNames.MAX_LINE_LENGTH, 15);        
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
+        params.setIntParameter(CoreConnectionPNames.MAX_LINE_LENGTH, 15);
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
         try {
             requestParser.fillBuffer(newChannel("GET / HTTP/1.0\r\nHeader: 9012345\r\n 23456789012345\r\n 23456789012345\r\n 23456789012345\r\n\r\n"));
             requestParser.parse();
@@ -400,11 +384,11 @@ public class TestHttpMessageParser extends TestCase {
 
     public void testMaxHeaderCount() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
 
         params.setIntParameter(CoreConnectionPNames.MAX_HEADER_COUNT, 2);
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
         requestParser.fillBuffer(newChannel("GET /whatever HTTP/1.0\r\nHeader: one\r\nHeader: two\r\n\r\n"));
         requestParser.parse();
         requestParser.reset();
@@ -419,11 +403,11 @@ public class TestHttpMessageParser extends TestCase {
 
     public void testDetectLineLimitEarly() throws Exception {
         HttpParams params = new BasicHttpParams();
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(2, 128, params); 
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(2, 128, params);
         HttpRequestFactory requestFactory = new DefaultHttpRequestFactory();
 
         params.setIntParameter(CoreConnectionPNames.MAX_LINE_LENGTH, 2);
-        HttpRequestParser requestParser = new HttpRequestParser(inbuf, null, requestFactory, params);
+        NHttpMessageParser<HttpRequest> requestParser = new DefaultHttpRequestParser(inbuf, null, requestFactory, params);
         ReadableByteChannel channel = newChannel("GET / HTTP/1.0\r\nHeader: one\r\n\r\n");
         assertEquals(2, requestParser.fillBuffer(channel));
         assertNull(requestParser.parse());
