@@ -58,7 +58,7 @@ public final class HttpHost implements Cloneable, Serializable {
     /** The port to use. */
     protected final int port;
 
-    /** The scheme */
+    /** The scheme (lowercased) */
     protected final String schemeName;
 
 
@@ -187,7 +187,6 @@ public final class HttpHost implements Cloneable, Serializable {
 
 
     public boolean equals(final Object obj) {
-        if (obj == null) return false;
         if (this == obj) return true;
         if (obj instanceof HttpHost) {
             HttpHost that = (HttpHost) obj;
